<?php
/***************************************************************************************************************************
****************************************************************************************************************************
****************************************************************************************************************************
***************                                                                                              ***************
***************            Clase para ordenacin de matrices en base a categoras.                           ***************
***************            Versin 1.2                                                                       ***************
***************            Desarrollada por Jos Lpez Quijado.		                                         ***************
***************            Es de libre uso, distribucin y modificacin.                                     ***************
***************                                                                                              ***************
****************************************************************************************************************************
****************************************************************************************************************************
***************************************************************************************************************************/

class ordenarCategorias{
// Se definen las propiedades que tendrn los objetos de esta clase.
	public $matriz = array(); //Matriz a procesar.
	public $idItem; //Cdigo de la categoria del elemento.
	public $nombreItem; //Nombre de la categoria.
	public $idPadre; //Codigo de la categoria de la que depende.
	public $nuevaPosicionGeneral = 0; //Indicador de posicion del elemento con respecto a otros.
	public $salidaTabulada = false; //Salida tabulada o con lista completa escalada de jerarquias (opcin por defecto).
	public $idPadreBase = 0; //La id de padre base puede cambiar, segn se establezca en la llamada.
	public $cadenaDeSeparacion = " > "; //Separacin entre elementos en la lista por jerarquas.
	public $tabulacion = "&nbsp;&nbsp;&nbsp;&nbsp;"; //Tabulacin por nivel, en la lista tabulada.

	/* El siguiente mtodo ajusta la id de padre de los registros a 0 en los de ms alto nivel,
si no es 0 previamente. */
	function idPadreCero(){
		foreach ($this->matriz as $clave=>$valor){
			if ($valor[$this->idPadre] == $this->idPadreBase) $this->matriz[$clave][$this->idPadre] = "0";
		}
	}

/* El siguiente mtodo recupera el id de padre original, por si no era 0. */ 
	function restaurarPadreBase(){
		foreach ($this->matriz as $clave=>$valor){
			if ($valor[$this->idPadre] == "0") $this->matriz[$clave][$this->idPadre] = $this->idPadreBase;
		}
	}

/* El siguiente mtodo aade un campo numrico a cada elemento de la matriz.
Este se emplear para realizar la ordenacin por categoras pero, de momento,
se inicializa a cero.*/
	function agregarPosVertical(){
		foreach ($this->matriz as $clave=>$valor){
			$this->matriz[$clave]["posVertical"] = 0;
		}
	}


/* El siguiente mtodo coloca valores en los campos numricos aadidos a la matriz, en base a las 
relaciones entre el cdigo de cada categora y el cdigo de aquella de la que desciende cada una. */
	function ponerPosiciones($categoriaPadre=0){
		$matrizNueva = array();
		foreach($this->matriz as $elemento){
			if ($elemento[$this->idPadre] == $categoriaPadre && $elemento["posVertical"] == 0){
				$matrizNueva[] = $elemento;
			}
		}

		foreach ($matrizNueva as $elemento){
			$this->nuevaPosicionGeneral ++;

			foreach ($this->matriz as $clave=>$item){
				if ($item[$this->idItem] == $elemento[$this->idItem]){
					$this->matriz[$clave]["posVertical"] = $this->nuevaPosicionGeneral;
				}
			}
			$argumento = (int)$elemento[$this->idItem];
			$this->ponerPosiciones($argumento);
		}
	}

/* Una vez colocados los valores nmericos en los campos que aadimos, se ordenan los
elementos de la matriz en base, precisamente, a dichos campos.*/
	function reOrdenar(){
		function ordenar($a, $b){
		    return ($a["posVertical"] < $b["posVertical"]) ? -1 : 1;
		}
		usort($this->matriz, "ordenar");
	}

/* Los nombres de las categorias se modifican de modo que cada uno incluya la secuencia
de todas las categoras de las que desciende, con un separador especfico. */
	function reconstruirNombres(){
		foreach ($this->matriz as $clave=>$elemento){
			if ($elemento[$this->nombreItem]!="0"){
				$categoriaBuscada = (int)$elemento[$this->idPadre];
				$cadenaPrecedente = "";
				while(true){
					foreach ($this->matriz as $item){
						if ($item[$this->idItem] == $categoriaBuscada){
							$cadenaPrecedente = $item[$this->nombreItem].$this->cadenaDeSeparacion.$cadenaPrecedente;
							$categoriaBuscada -= (int)$item[$this->idItem];
						}
					}
					if ($categoriaBuscada == 0) break;
				}
				$this->matriz[$clave][$this->nombreItem] = $cadenaPrecedente.$elemento[$this->nombreItem];
			}
		}
	}

/* Por ltimo, se eliminan los campos que se aadieron, y que ya no son necesarios. */
	function eliminarPosVertical(){
		foreach ($this->matriz as $clave=>$valor){
			unset($this->matriz[$clave]["posVertical"]);
		}
	}

/* El mtodo crearMapa() encadena la llamada a los distintos mtodos de la clase,
en la secuencia adecuada, y devuelve la matriz ordenada segn nuestros deseos. */
	function crearMapa(){
		$this->idPadreCero();
		$this->agregarPosVertical();
		$this->ponerPosiciones();
		$this->reconstruirNombres();
		$this->reOrdenar();
		$this->eliminarPosVertical();
		if($this->salidaTabulada){ //Si se quiere la salida con tabulaciones, esta se establece aqu.
			foreach($this->matriz as $indice=>$elemento){
				$tabulacion = "";
				while(true){
					$recorte = strstr($elemento[$this->nombreItem], $this->cadenaDeSeparacion);
					if (!$recorte) break;
					$elemento[$this->nombreItem] = substr($recorte, strlen($this->cadenaDeSeparacion));
					$tabulacion .= $this->tabulacion;
				}
				$this->matriz[$indice][$this->nombreItem] = $tabulacion.$elemento[$this->nombreItem];
			}
		}
		$this->restaurarPadreBase();
		return $this->matriz;
	}
}
?>