<?php
/* La siguiente funcin recorre la carpeta 
establecida como $rutaDeImagenes y detecta 
las imgenes que hay.
Luego las miniaturiza, segn el tamao especificado 
y las graba en la carpeta al efecto,
respetando los originales.
ATENCIN!!! Las imgenes solo pueden ser de 
tipo JPG, GIF o PNG (los formatos empleados en Internet). */
	function miniaturizar($rutaDeImagenes, $rutaDeMiniaturas, $maxAnchoMini, $maxAltoMini){	
		$matrizDeImagenes = array();
		$matrizDeImagenesAReducir = array();

/* Se abre el directorio especificado como de 
imgenes originales y se recupera en una matriz 
una lista de todos los archivos que hay dentro. */
		$manejadorDeImagenes = opendir($rutaDeImagenes);
		rewinddir($manejadorDeImagenes);
		while ($imagen = readdir($manejadorDeImagenes)) $matrizDeImagenes[] = $imagen;
		closedir($manejadorDeImagenes);

/* Se recorre la matriz de nombres de las imagenes originales 
y se copian, en otra matriz, las que no sean excluidas.
La exlusin es causada por:
	- Si el nombre no corresponde a un fichero (por ejemplo, 
	  la cabecera de directorio).
	- Si en la carpeta de miniaturas ya existe un archivo con 
	  el mismo nombre.
	- Si el archivo no puede obtenerse como imagen.
	- Si el tipo MIME del archivo no es coincidente 
	  con una imagen.
	- Si es una imagen pero no es JPG, GIF ni PNG 
	  (p.e. si es BMP). */
		foreach ($matrizDeImagenes as $imagen){
			$imagenOriginal = $rutaDeImagenes.$imagen;
			if (! is_file($imagenOriginal)) continue; // No es un fichero.
			$miniatura = $rutaDeMiniaturas.$imagen;
			if (file_exists($miniatura)) continue; // Ya existe la miniatura.
			if (! @getimagesize($imagenOriginal)) continue; // No puede obtenerse como imagen.
			$datos = @getimagesize($imagenOriginal);
			$tipo = $datos["mime"];
			if (strpos($tipo, "image") < 0) continue; // El tipo MIME no se reconoce como imagen.
			$tipo = substr($datos["mime"], 6);
			if ($tipo != "jpeg" && $tipo != "jpg" && $tipo != "png" && $tipo != "gif") continue; // Es una imagen no vlida.
			$matrizDeImagenesAReducir[] = $imagen; // Se acepta la imagen para su miniaturizacin.
		}
		unset ($matrizDeImagenes); // Se elimina la matriz previa, que ya no es necesaria.

/* Se recorre la matriz de nombres de imgenes aceptadas 
para su miniaturizacin, llevando a cabo los siguientes 
procesos:
	- Se obtiene su anchura y su altura.
	- Se calcula el ratio (la proporcin) en anchura 
	  y altura con respoecto a los lmites mximos 
	  pasados a la funcin para las miniaturas.
	- Se calculan las dimensiones que deber tener cada 
	  miniatura en base al ratio ms restrictivo, de modo 
	  que nunca excedan de las dimensiones mximas especificadas 
	  y mantengan las proporciones de la imagen original, para 
	  que no resulten deformadas.
	- Se crea un recurso "en blanco" para la imagen en 
	  miniatura, con las dimensiones que se han determinado.
	- Se crea un recurso de la imagen original, acorde con su tipo MIME.
	- Se hace una copia remuestreada de la original en la miniatura.
	- Se graba la miniatura en la correspondiente carpeta, con 
	  el mismo nombre que la original, segn su tipo MIME.*/
		foreach ($matrizDeImagenesAReducir as $clave=>$imagen){
			$imagenOriginal = $rutaDeImagenes.$imagen;
			$imagenMiniatura = $rutaDeMiniaturas.$imagen;
			$datos = getimagesize($imagenOriginal);
			$ratioAncho = $datos[0]/$maxAnchoMini;
			$ratioAlto = $datos[1]/$maxAltoMini;
			if ($ratioAncho <= $ratioAlto){
				$anchoMini = (int)(floor($datos[0]/$ratioAlto));
				$altoMini = (int)(floor($datos[1]/$ratioAlto));
			} else {
				$anchoMini = (int)(floor($datos[0]/$ratioAncho));
				$altoMini = (int)(floor($datos[1]/$ratioAncho));
			}

			$recursoMiniatura = imagecreatetruecolor($anchoMini, $altoMini);
			if ($tipo == "jpeg" || $tipo == "jpg"){ // Imagen JPG
				$recursoOriginal = imagecreatefromjpeg($imagenOriginal);
			} elseif ($tipo == "png"){ // Imagen PNG
				$recursoOriginal = imagecreatefrompng($imagenOriginal);
			} else { // Imagen GIF
				$recursoOriginal = imagecreatefromgif($imagenOriginal);
			}

			imagecopyresampled ($recursoMiniatura, $recursoOriginal, 0, 0, 0, 0, $anchoMini, $altoMini, $datos[0], $datos[1]);

			if ($tipo == "jpeg" || $tipo == "jpg"){ // Imagen JPG
				imagejpeg($recursoMiniatura, $imagenMiniatura);
			} elseif ($tipo == "png"){ // Imagen PNG
				imagepng($recursoMiniatura, $imagenMiniatura);
			} else { // Imagen GIF
				imagegif($recursoMiniatura, $imagenMiniatura);
			}
		}
	}
?>
